/*
 Copyright 2011 Andres Leonardo Martinez Ortiz

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */
import java.io.InputStreamReader;
import java.io.BufferedReader;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.signature.HmacSha1MessageSigner;

import java.net.HttpURLConnection;
import java.net.URL;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author almo
 */
public class BVSMSCollector extends Thread {

  private boolean bRun;
  private moRevenueCalGUI _moRevenueCalGUI;
  private OAuthConsumer _apiConsumer;
  private String _registrationID;

  public BVSMSCollector(moRevenueCalGUI moRevenueCalGUI, OAuthConsumer apiConsumer, String registrationID){
    bRun=true;
    _moRevenueCalGUI = moRevenueCalGUI;
    _apiConsumer=apiConsumer;
    _registrationID = registrationID;
  }

  public synchronized void finish(){
      bRun=false;
  }

  public synchronized boolean isRunning(){
      return bRun;
  }

  @Override
  public void run(){

    HttpURLConnection request = null;
    URL moAPIurl = null;

	JSONObject apiResponse;
    JSONObject smsPool;
    JSONArray  smsInfo;
		
    int rc=0;
    double revenue=0.0;
 
      try{
    	moAPIurl = new URL("https://api.bluevia.com/services/REST/SMS_Sandbox/inbound/"+ _registrationID +"/messages?version=v1&alt=json");
        do{
            request = (HttpURLConnection) moAPIurl.openConnection();
			request.setRequestMethod("GET");
			request.setRequestProperty("Accept", "application/json");

            _apiConsumer.sign(request);
			
			StringBuffer doc = new StringBuffer();
			BufferedReader br =null;
			
			rc = request.getResponseCode();
			if (rc == HttpURLConnection.HTTP_OK)
				br = new BufferedReader(new InputStreamReader(request.getInputStream()));
			else
				br = new BufferedReader(new InputStreamReader(request.getErrorStream()));
		
            String line = br.readLine();
            while (line != null) {
            	doc.append(line);
                line = br.readLine();
            }

            apiResponse = new JSONObject(doc.toString());
            
            if (apiResponse.getString("receivedSMS")!=null){            
                smsPool = apiResponse.getJSONObject("receivedSMS");  
                smsInfo = smsPool.getJSONArray("receivedSMS");
            
                String szMessage=null;
                String szOrigin=null;
                String szDate=null;
                    	        
            	for (int i=0; i<smsInfo.length(); i++){
            		szMessage = smsInfo.getJSONObject(i).getString("message");
            		szOrigin = smsInfo.getJSONObject(i).getJSONObject("originAddress").getString("phoneNumber");
            		szDate = smsInfo.getJSONObject(i).getString("dateTime");
                    _moRevenueCalGUI.appendSMSLog(szDate +": "+"From: "+szOrigin+" -> "+szMessage);

            	    revenue+= 0.03;
                    _moRevenueCalGUI.setRevenue(revenue);
                }
             }

            request.disconnect();
 
           sleep(2000);  

         }while (isRunning());
     }catch (InterruptedException e){
          System.err.println(e.getMessage());
     }catch (JSONException e){
		  System.err.println(e.getMessage());
		  request.disconnect();
	 }catch (Exception e){
	  	  System.err.println(e.getMessage());			
	 }finally {
        request.disconnect();
     }
  }
}
