/*
 Copyright 2010 Andres Leonardo Martinez Ortiz

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.signature.HmacSha1MessageSigner;

import java.net.HttpURLConnection;
import java.net.URL;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * @author almo
 * You can get more examples at anonymous svn:
 * svn checkout https://svn.forge.morfeo-project.org/insomne/trunk/Mobile-Clients/examples
 * You can generate a lot of funny messages with the following shell script command:
 * while test true; do java -cp .:lib/* mtSMSDemo "`fortune -s | tr '\n' ' '|tr '"' ' '`"; sleep 5; done
 */
public class moSMSDemo {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws IOException{

		BufferedReader iReader = null;
		String apiDataFile = "API-AccessToken.ini";
		
		String consumer_key;
		String consumer_secret;
		String apiAccessToken_key;
		String apiAccessToken_secret;

		String registrationId;
		
		OAuthConsumer apiConsumer = null;
		HttpURLConnection request = null;
		URL moAPIurl = null;
		
		JSONObject apiResponse;
		JSONObject smsPool;
		JSONArray  smsInfo;
		
		int rc=0;
		
		try{
			iReader = new BufferedReader (new FileReader(apiDataFile));
			
			// Private data:  consumer info + access token info + phone info
			consumer_key = iReader.readLine();
        	consumer_secret= iReader.readLine();
			apiAccessToken_key = iReader.readLine();
			apiAccessToken_secret = iReader.readLine();
			registrationId = iReader.readLine();			
			
			// Set up the oAuthConsumer
			apiConsumer = new DefaultOAuthConsumer(consumer_key,consumer_secret);
			apiConsumer.setMessageSigner(new HmacSha1MessageSigner ());
			apiConsumer.setTokenWithSecret(apiAccessToken_key, apiAccessToken_secret);
			
			moAPIurl = new URL("https://api.bluevia.com/services/REST/SMS_Sandbox/inbound/"+ registrationId +"/messages?version=v1&alt=json");
			
			request = (HttpURLConnection) moAPIurl.openConnection();
			request.setRequestMethod("GET");
			request.setRequestProperty("Accept", "application/json");
			
			apiConsumer.sign(request);
			
			StringBuffer doc = new StringBuffer();
			BufferedReader br =null;
			
			rc = request.getResponseCode();
			if (rc == HttpURLConnection.HTTP_OK)
				br = new BufferedReader(new InputStreamReader(request.getInputStream()));
			else
				br = new BufferedReader(new InputStreamReader(request.getErrorStream()));
            
			String line = br.readLine();
            while (line != null) {
            	doc.append(line);
                line = br.readLine();
            }
            
            System.out.printf("Output message: %s\n", doc.toString());
            
            apiResponse = new JSONObject(doc.toString());
            if (apiResponse.getString("receivedSMS")!=null){
            	smsPool = apiResponse.getJSONObject("receivedSMS");  
            	smsInfo = smsPool.getJSONArray("receivedSMS");
            
            	String szMessage;
            	String szOrigin;
            	String szDate;
                    	        
            	for (int i=0; i<smsInfo.length(); i++){
            		szMessage = smsInfo.getJSONObject(i).getString("message");
            		szOrigin = smsInfo.getJSONObject(i).getJSONObject("originAddress").getString("phoneNumber");
            		szDate = smsInfo.getJSONObject(i).getString("dateTime");
        	
            		System.out.printf("Message #%d: %s\nFrom: %s\nOn: %s\n\n",i, szMessage,szOrigin,szDate);
            	}
            }else
            	System.out.printf("No messages\n");
            
			request.disconnect();
			
		} catch (JSONException e){
			System.err.println(e.getMessage());
			request.disconnect();
		} catch (Exception e){
			System.err.println(e.getMessage());			
		} finally {
			if (iReader!=null)
				iReader.close();
		}
	}

}
