/*
 Copyright 2010 Andres Leonardo Martinez Ortiz

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;

import java.net.HttpURLConnection;
import java.net.URL;

import oauth.signpost.OAuthConsumer;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.signature.HmacSha1MessageSigner;

/**
 * 
 * @author almo
 * This demo shows how to send an sms using the BlueVia API. IMPORTANT: you will need a 
 * valid access token. You can see how to get one taking a look at oAuthDemo. 
 * You can get more examples at anonymous svn:
 * svn checkout https://svn.forge.morfeo-project.org/insomne/trunk/Mobile-Clients/examples
 */
public class mtSMSDemo {

	/**
	 * @param args
	 */
	public static void main(String[] args)  throws IOException{
		
		BufferedReader iReader = null;
		String apiDataFile = "API-AccessToken.ini";
		
		String smsJsonMessage = "{\"smsText\": {\n  \"address\": {\"phoneNumber\": \"%s\"},\n  \"message\": \"%s\",\n  \"originAddress\": {\"alias\": \"%s\"},\n}}";
		
		String consumer_key;
		String consumer_secret;
		String apiAccessToken_key;
		String apiAccessToken_secret;
		
		String dstPhoneNumber;
		String txtMessage="SANDtwitter Stop making sense, just send me an angel...";
		
		OAuthConsumer apiConsumer = null;
		
		HttpURLConnection request = null;
		URL smsAPIurl = null;
		String smsJsonMsg;
		String deliveryStatusURL;
		Integer rc;
		
		System.out.println("Starting mtSMSDemo....");
		
		try{
			iReader = new BufferedReader (new FileReader(apiDataFile));
			
			// Private data:  consumer info + access token info + phone info
			consumer_key = iReader.readLine();
        	consumer_secret= iReader.readLine();
			apiAccessToken_key = iReader.readLine();
			apiAccessToken_secret = iReader.readLine();
			dstPhoneNumber = iReader.readLine();
			
			// Customization with fortune
			if (args.length == 1)
				txtMessage = "SANDtwitter " + args[0];
			
			// Set up the oAuthConsumer
			apiConsumer = new DefaultOAuthConsumer(consumer_key,consumer_secret);
			apiConsumer.setMessageSigner(new HmacSha1MessageSigner ());
			apiConsumer.setTokenWithSecret(apiAccessToken_key, apiAccessToken_secret);
			
			smsAPIurl = new URL("https://api.bluevia.com/services/REST/SMS_Sandbox/outbound/requests?version=v1"); 
			request = (HttpURLConnection) smsAPIurl.openConnection();
	        
	        request.setDoOutput(true);
			request.setRequestMethod("POST");
            request.setRequestProperty("Content-Type", "application/json");
            
            apiConsumer.sign(request);			
            
            // Creating JSON message
            smsJsonMsg = String.format(smsJsonMessage+"\n", dstPhoneNumber, txtMessage, apiAccessToken_key);
            System.out.println(smsJsonMsg);
            
            // Posting the message
            OutputStream os = request.getOutputStream();           
			os.write(smsJsonMsg.getBytes()); 
			os.flush();
						
	        System.out.printf("Sending to %s\nthe SMS %s (%d characters)\n",dstPhoneNumber, txtMessage, txtMessage.length());
	        
	        request.connect();
	        rc = request.getResponseCode();
	        System.out.printf("%d : %s\n",rc ,request.getResponseMessage());
	        
	        if (rc == HttpURLConnection.HTTP_CREATED){
	        	deliveryStatusURL = request.getHeaderField("Location");
	        	System.out.printf("SMS is sent. \nYou can trace its delivery at %s\n",deliveryStatusURL);
	        }
	        
		}catch (Exception e){
			System.err.println(e.getMessage());			
		} finally {
			if (iReader!=null)
				iReader.close();
		}
		
		System.out.println("Finishing mtSMSDemo....");
	}
}