#
# BlueVia is a global iniciative of Telefonica delivered by Movistar and O2.
# Please, check out www.bluevia.com and if you need more information
# contact us at mailto:support@bluevia.com

module Bluevia
  #
  # This module is a mixin used by SMS client to get some basic messaging features
  #

  module Messaging


    #
    # This method is used to retrieve MO messages that belongs to a specific registration ID
    # [*registration_id*] MO Registration Identifier Mandatory
    # [*message_id*] message unique identifier
    # [*attachments*] bool flag to get message attachments (if true)
    # returns a collection of messages
    #
    def get_received_message(registration_id, message_id = nil, attachments = false)
      Utils.check_attribute(registration_id, "Provisioning registration id must be provided")
      unless message_id.nil?
        message_id = "/#{message_id}"
        unless attachments == false
          attachments = "/attachments"
        else
          attachments = ""
        end
      else
        message_id = ""
        attachments = ""
      end
      GET("#{get_basepath}/inbound/#{registration_id}/messages#{message_id}#{attachments}")
    end

    #
    # This method allows to retrieved a specific attachment belonging to a message
    # [*registration_id*] MO Registration Identifier
    # [*message_id*] message unique identifier
    # [*attachment_id*] attachment identifier (corresponding to the specific message
    #

    def get_attachment(registration_id, message_id, attachment_id)
      Utils.check_attribute(registration_id, "Registration id must be provided")
      Utils.check_attribute(message_id, "Message id must be provided")
      Utils.check_attribute(attachment_id, "Attachment id must be provided")
      GET("#{get_basepath}/inbound/#{registration_id}/messages/#{message_id}/attachments/#{attachment_id}")
    end

    #
    # This method returns the delivery status of a SMS sent by the user
    # [*id*] message unique identifier
    #

    def get_delivery_status (id)
      Utils.check_attribute id, "#{self.class.to_s.upcase} identifier cannot be null"
      if is_url?(id)
        id = id.split("/").last(2).first
      end
      response = GET("#{get_basepath}/outbound/requests/#{id}/deliverystatus", nil, nil)

      # Clean duplicated key
      if response.instance_of?(Hash)
        if response.has_key?("smsDeliveryStatus")
          return response["smsDeliveryStatus"]
        elsif response.has_ley?("messageDeliveryStatus")
          return response["messageDeliveryStatus"]
        else
          return response
        end
      end
    end

    #
    # This method stop the message notification (MO)
    # [*subscription_id*] The subscription identifier to be deleted
    #

    private

    #
    # Validates if a URL is defined properly
    #
    def is_url?(identifier)
      if identifier.match(/^https?:\/\//i).nil?
      # if identifier.match(/^https?:\/\/([a-z0-9-]+\.)+[a-z0-9]+(:[0-9]{2,5})?/i)
      #if identifier.match(/^https?:\/\/([a-z]+\.)*[a-z]+$/i).nil?
      #if identifier.match(/^https?:\/\/([a-z]+\.)*[a-z]+$/i).nil?
        return false
      else
        return true
      end
    end
  end
end
