#
# BlueVia is a global iniciative of Telefonica delivered by Movistar and O2.
# Please, check out www.bluevia.com and if you need more information
# contact us at mailto:support@bluevia.com

require 'bluevia/messaging'
require 'bluevia/schemas/sms_types'

module Bluevia
  #
  # This class is in charge of access Bluevia SMS API
  #

  class Sms < BaseClient
    include Messaging

    include Schemas::Sms_types

    # Basepath for SMS API
    BASEPATH_API = "/SMS"

    def initialize(params = nil)
      super(params)
      #super.extend(Schemas::Sms_types)
    end

    # 
    # This method sends a SMS to a list of destination numbers.
    # [*dest_number*] array, fixnum or string with the destination numbers
    # [*origin*] originator number the SMS will be send in name of. Should be a
    # valid access_token.
    # [*text*] short message text
    # returns the location where the delivery status can be checked
    # 
    def send_sms (dest_number, origin, text)
      Utils.check_attribute dest_number, "Destination number cannot be null"
      Utils.check_attribute text, "SMS text cannot be null"
      Utils.check_attribute origin, "Origin number cannot be null"

      # Set destination array
      @dest_phones = Array.new
      if (dest_number.instance_of?(Array))
        dest_number.each { |i|
          @dest_phones << UserIdType.new(i, nil, nil, nil, nil)
        }
      else
        @dest_phones << UserIdType.new(dest_number, nil, nil, nil, nil)
      end

      @orig = UserIdType.new(nil, nil, nil, origin, nil)

      # Create Message and convert to JSON
      @msg = SMSTextType.new(@dest_phones, text, nil, @orig, nil, nil, nil, nil, nil)
      message = Utils.object_to_json(@msg)

      logger.debug "SMS Send this message:"
      logger.debug message << "\n"

      POST("#{get_basepath}/outbound/requests", message.to_s, nil, nil, "location")
    end

    #
    # This method is used to retrieve MO messages that belongs to a specific registration ID
    # [*registration_id*] MO Registration Identifier Mandatory
    # returns a collection of messages
    #
    def get_received_sms(registration_id)
      response = get_received_message(registration_id)
      key = "receivedSMS"

      # Fetch ["receivedSMS"]["receivedSMS"]
      unless response.nil?
        if response.respond_to?(:has_key?) && response.has_key?(key)
          response = response[key]
          if response.respond_to?(:has_key?) && response.has_key?(key)
            return response[key]
          else
            return Hash.new
          end
        else
          return Hash.new
        end
      else
        return Hash.new
      end
      
    end


  end
end
