#
# BlueVia is a global iniciative of Telefonica delivered by Movistar and O2.
# Please, check out www.bluevia.com and if you need more information
# contact us at mailto:support@bluevia.com

$:.unshift File.join(File.dirname(__FILE__),'..','lib')

require 'test/unit'
require 'bluevia'
require 'bluevia/errors'
require 'test_config'

#
# (c) Bluevia (mailto:support@bluevia.com)
#

class TestSms < Test::Unit::TestCase
  include TestConfig
  include Bluevia

  URL_DELIVERY_NOTIFICATION = "http://foo.bar"

  def setup
    create_client(false, :Sms)
  end

  def test_send_sms
    send_sms(TestConfig::SandBox::TOKEN)
  end

  def test_send_sms_commercial
    create_client(true, :Sms)
    send_sms(TestConfig::Commercial::TOKEN)
  end

  def test_get_delivery_status
    get_delivery_status(TestConfig::SandBox::TOKEN)
  end

  def test_get_delivery_status_commercial
    create_client(true, :Sms)
    get_delivery_status(TestConfig::Commercial::TOKEN)
  end

  def test_get_delivery_status_commercial_identifier
    create_client(true, :Sms)
    get_delivery_status(TestConfig::Commercial::TOKEN, "10001112090452109328")
  end

  def ftest_get_received_sms
    get_received_sms
  end

  def ftest_get_received_sms_fails
    get_received_sms_fails
  end

  def ftest_get_received_sms_commercial
    create_client(true, :Sms)
    get_received_sms(true)
  end

  def ftest_get_received_sms_commercial_fails
    create_client(true, :Sms)
    get_received_sms_fails
  end

  private

  def get_correlator_id
    (0...10).map{ ('a'..'z').to_a[rand(26)] }.join
  end

  def send_sms(origin)
    info = @service.send_sms(["11111", "33333"], origin, "Yet another SMS Text")
    assert_not_nil info
    assert_match(/^.*(deliverystatus)$/, info)
    info
  end

  def get_received_sms(commercial = false)
    if commercial
      registration_id = "4353432" # TODO: change this number
    else
      registration_id = "35234264"
    end
    info = @service.get_received_sms(registration_id)
    assert_equal info.include?("receivedSMS"), true
    p info
  end

  def get_received_sms_fails
    assert_raise RuntimeError do
      @service.get_received_sms("352342641")
    end
  end

  def get_delivery_status(origin, identifier = nil)
    if identifier.nil?
      info = send_sms(origin)
      size = 2
      identifier = info.split("/").last(2).first
    else
      size = 1
    end
    info = @service.get_delivery_status(identifier)
    assert_equal info["smsDeliveryStatus"].size, size
    assert_equal info["smsDeliveryStatus"][0].include?("address"), true
    assert_equal info["smsDeliveryStatus"][0].include?("deliveryStatus"), true
  end

end
